@@ -11,7 +11,7 @@ test tree
 """
 import unittest
 
-from sqlalchemy import Column, create_engine, Integer, Boolean
+from sqlalchemy import Boolean, Column, create_engine, Integer
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import sessionmaker
 
@@ -1125,3 +1125,306 @@ class TestTree(unittest.TestCase):
                           (20, 14, 15, 4, 19, 2),
                           (21, 17, 20, 3, 18, 2),
                           (22, 18, 19, 4, 21, 2)])
+
+    def test_tree_shorting(self):
+        """ level           Nested sets example
+            1                    1(1)22
+                    _______________|___________________
+                   |               |                   |
+            2    2(2)5           6(4)11             12(7)21
+                   |               ^                   ^
+            3    3(3)4       7(5)8   9(6)10    13(8)16   17(10)20
+                                                  |          |
+            4                                  14(9)15   18(11)19
+
+            level           Nested sets example
+
+                                    __parent_id______________________
+                                   |                                 |
+            1                    1(1)22                              |
+                    _______________|___________________              |
+                   |               |                   |             |
+            2    2(2)5           6(4)11             12(7)21         (X)
+                   |               ^                   ^             |
+            3    3(3)4       7(5)8   9(6)10    13(8)16   17(10)20    |
+                                                  |          |       |
+            4                                  14(9)15   18(11)19    |
+                                                            ↑        |
+                                                            ↑________|
+                        id lft rgt lvl parent tree
+        """
+        node = self.session.query(Tree).filter(Tree.id == 1).one()
+        node.parent_id = 11
+        self.session.add(node)
+        self.assertEqual([(1,   1, 22, 1, None, 1),
+                          (2,   2,  5, 2,  1, 1),
+                          (3,   3,  4, 3,  2, 1),
+                          (4,   6, 11, 2,  1, 1),
+                          (5,   7,  8, 3,  4, 1),
+                          (6,   9, 10, 3,  4, 1),
+                          (7,  12, 21, 2,  1, 1),
+                          (8,  13, 16, 3,  7, 1),
+                          (9,  14, 15, 4,  8, 1),
+                          (10, 17, 20, 3,  7, 1),
+                          (11, 18, 19, 4, 10, 1),
+
+                          (12,  1, 22, 1, None, 2),
+                          (13,  2,  5, 2, 12, 2),
+                          (14,  3,  4, 3, 13, 2),
+                          (15,  6, 11, 2, 12, 2),
+                          (16,  7,  8, 3, 15, 2),
+                          (17,  9, 10, 3, 15, 2),
+                          (18, 12, 21, 2, 12, 2),
+                          (19, 13, 16, 3, 18, 2),
+                          (20, 14, 15, 4, 19, 2),
+                          (21, 17, 20, 3, 18, 2),
+                          (22, 18, 19, 4, 21, 2)], self.result.all())
+
+    def test_move_inside_to_the_same_parent_function(self):
+        """ level           Nested sets example
+                1                    1(1)22
+                        _______________|___________________
+                       |               |                   |
+                2    2(2)5           6(4)11             12(7)21
+                       |               ^                   ^
+                3    3(3)4       7(5)8   9(6)10    13(8)16   17(10)20
+                                                      |          |
+                4                                  14(9)15   18(11)19
+
+            level           move 6 inside 4
+                1                    1(1)22
+                        _______________|___________________
+                       |               |                   |
+                2    2(2)5           6(4)11             12(7)21
+                       |               ^                   ^
+                3    3(3)4       7(6)8   9(5)10    13(8)16   17(10)20
+                                                      |          |
+                4                                  14(9)15   18(11)19
+
+                          id lft rgt lvl parent tree
+        """
+        node = self.session.query(Tree).filter(Tree.id == 6).one()
+        node.move_inside("4")
+        self.assertEqual([(1,   1, 22, 1, None, 1),
+                          (2,   2,  5, 2,  1, 1),
+                          (3,   3,  4, 3,  2, 1),
+                          (4,   6, 11, 2,  1, 1),
+                          (5,   9, 10, 3,  4, 1),
+                          (6,   7,  8, 3,  4, 1),
+                          (7,  12, 21, 2,  1, 1),
+                          (8,  13, 16, 3,  7, 1),
+                          (9,  14, 15, 4,  8, 1),
+                          (10, 17, 20, 3,  7, 1),
+                          (11, 18, 19, 4, 10, 1),
+
+                          (12,  1, 22, 1, None, 2),
+                          (13,  2,  5, 2, 12, 2),
+                          (14,  3,  4, 3, 13, 2),
+                          (15,  6, 11, 2, 12, 2),
+                          (16,  7,  8, 3, 15, 2),
+                          (17,  9, 10, 3, 15, 2),
+                          (18, 12, 21, 2, 12, 2),
+                          (19, 13, 16, 3, 18, 2),
+                          (20, 14, 15, 4, 19, 2),
+                          (21, 17, 20, 3, 18, 2),
+                          (22, 18, 19, 4, 21, 2)], self.result.all())
+
+    def test_move_before_to_top_level(self):
+        """ level           Nested sets example
+                1                    1(1)22
+                        _______________|___________________
+                       |               |                   |
+                2    2(2)5           6(4)11             12(7)21
+                       |               ^                   ^
+                3    3(3)4       7(5)8   9(6)10    13(8)16   17(10)20
+                                                      |          |
+                4                                  14(9)15   18(11)19
+
+            level            move 4 before 1
+                1         1(4)6              1(1)16
+                            ^           _______|_______
+                      2(5)3   4(6)5    |               |
+                2                    2(2)5           6(7)15
+                                       |               ^
+                3                    3(3)4      7(8)10   11(10)14
+                                                  |          |
+                4                               8(9)9    12(11)13
+
+                          id lft rgt lvl parent tree
+        """
+        node = self.session.query(Tree).filter(Tree.id == 4).one()
+        node.move_before("1")
+        self.maxDiff = None
+        self.assertEqual([(1,   1, 16, 1, None, 2),
+                          (2,   2,  5, 2,  1, 2),
+                          (3,   3,  4, 3,  2, 2),
+
+                          (4,   1,  6, 1,  None, 1),
+                          (5,   2,  3, 2,  4, 1),
+                          (6,   4,  5, 2,  4, 1),
+
+                          (7,   6, 15, 2,  1, 2),
+                          (8,   7, 10, 3,  7, 2),
+                          (9,   8,  9, 4,  8, 2),
+                          (10, 11, 14, 3,  7, 2),
+                          (11, 12, 13, 4, 10, 2),
+
+                          (12,  1, 22, 1, None, 3),
+                          (13,  2,  5, 2, 12, 3),
+                          (14,  3,  4, 3, 13, 3),
+                          (15,  6, 11, 2, 12, 3),
+                          (16,  7,  8, 3, 15, 3),
+                          (17,  9, 10, 3, 15, 3),
+                          (18, 12, 21, 2, 12, 3),
+                          (19, 13, 16, 3, 18, 3),
+                          (20, 14, 15, 4, 19, 3),
+                          (21, 17, 20, 3, 18, 3),
+                          (22, 18, 19, 4, 21, 3)], self.result.all())
+
+    def test_move_before(self):
+        """ level           Nested sets example
+            1                    1(1)22
+                    _______________|___________________
+                   |               |                   |
+            2    2(2)5           6(4)11             12(7)21
+                   |               ^                   ^
+            3    3(3)4       7(5)8   9(6)10    13(8)16   17(10)20
+                                                  |          |
+            4                                  14(9)15   18(11)19
+
+            level           move 8 before 4
+            1                    1(1)22
+                    _______________|___________________
+                   |        |            |             |
+            2    2(2)5    6(8)9       10(4)15       16(7)21
+                   |        |            ^             |
+            3    3(3)4    7(9)8   11(5)12 13(6)14  17(10)20
+                                                       |
+            4                                      18(11)19
+
+                        id lft rgt lvl parent tree
+        """
+        node = self.session.query(Tree).filter(Tree.id == 8).one()
+        node.move_before("4")
+        self.assertEqual([(1,   1, 22, 1, None, 1),
+                          (2,   2,  5, 2,  1, 1),
+                          (3,   3,  4, 3,  2, 1),
+                          (4,  10, 15, 2,  1, 1),
+                          (5,  11, 12, 3,  4, 1),
+                          (6,  13, 14, 3,  4, 1),
+                          (7,  16, 21, 2,  1, 1),
+                          (8,   6,  9, 2,  1, 1),
+                          (9,   7,  8, 3,  8, 1),
+                          (10, 17, 20, 3,  7, 1),
+                          (11, 18, 19, 4, 10, 1),
+
+                          (12,  1, 22, 1, None, 2),
+                          (13,  2,  5, 2, 12, 2),
+                          (14,  3,  4, 3, 13, 2),
+                          (15,  6, 11, 2, 12, 2),
+                          (16,  7,  8, 3, 15, 2),
+                          (17,  9, 10, 3, 15, 2),
+                          (18, 12, 21, 2, 12, 2),
+                          (19, 13, 16, 3, 18, 2),
+                          (20, 14, 15, 4, 19, 2),
+                          (21, 17, 20, 3, 18, 2),
+                          (22, 18, 19, 4, 21, 2)], self.result.all())
+
+    def test_move_one_tree_before_other_tree(self):
+        node = self.session.query(Tree).filter(Tree.id == 12).one()
+        node.move_before("1")
+        self.assertEqual([(1,   1, 22, 1, None, 2),
+                          (2,   2,  5, 2,  1, 2),
+                          (3,   3,  4, 3,  2, 2),
+                          (4,   6, 11, 2,  1, 2),
+                          (5,   7,  8, 3,  4, 2),
+                          (6,   9, 10, 3,  4, 2),
+                          (7,  12, 21, 2,  1, 2),
+                          (8,  13, 16, 3,  7, 2),
+                          (9,  14, 15, 4,  8, 2),
+                          (10, 17, 20, 3,  7, 2),
+                          (11, 18, 19, 4, 10, 2),
+
+                          (12,  1, 22, 1, None, 1),
+                          (13,  2,  5, 2, 12, 1),
+                          (14,  3,  4, 3, 13, 1),
+                          (15,  6, 11, 2, 12, 1),
+                          (16,  7,  8, 3, 15, 1),
+                          (17,  9, 10, 3, 15, 1),
+                          (18, 12, 21, 2, 12, 1),
+                          (19, 13, 16, 3, 18, 1),
+                          (20, 14, 15, 4, 19, 1),
+                          (21, 17, 20, 3, 18, 1),
+                          (22, 18, 19, 4, 21, 1)], self.result.all())
+
+    def test_move_before_to_other_tree(self):
+        """ level           Move 8 before 15
+            1                    1(1)18
+                     _______________|___________________
+                    |               |                   |
+            2     2(2)5           6(4)11             12(7)17
+                    |               ^                   |
+            3     3(3)4       7(5)8   9(6)10        13(10)16
+                                                        |
+            4                                       14(11)15
+
+            level
+            1                    1(12)26
+                     _______________|______________________________
+                    |         |               |                    |
+            2    2(13)5     6(8)9         10(15)15             16(18)25
+                    |         |               ^                    ^
+            3    3(14)4     7(9)8    11(16)12  13(17)14   17(19)20   21(21)24
+                                                              |          |
+            4                                             18(20)19   22(22)23
+
+                        id lft rgt lvl parent tree
+        """
+        node = self.session.query(Tree).filter(Tree.id == 8).one()
+        node.move_before("15")
+        self.assertEqual([(1,   1, 18, 1, None, 1),
+                          (2,   2,  5, 2,  1, 1),
+                          (3,   3,  4, 3,  2, 1),
+                          (4,   6, 11, 2,  1, 1),
+                          (5,   7,  8, 3,  4, 1),
+                          (6,   9, 10, 3,  4, 1),
+                          (7,  12, 17, 2,  1, 1),
+
+                          (8,   6,  9, 2,  12, 2),
+                          (9,   7,  8, 3,   8, 2),
+
+                          (10, 13, 16, 3,  7, 1),
+                          (11, 14, 15, 4, 10, 1),
+
+                          (12,  1, 26, 1, None, 2),
+                          (13,  2,  5, 2, 12, 2),
+                          (14,  3,  4, 3, 13, 2),
+                          (15, 10, 15, 2, 12, 2),
+                          (16, 11, 12, 3, 15, 2),
+                          (17, 13, 14, 3, 15, 2),
+                          (18, 16, 25, 2, 12, 2),
+                          (19, 17, 20, 3, 18, 2),
+                          (20, 18, 19, 4, 19, 2),
+                          (21, 21, 24, 3, 18, 2),
+                          (22, 22, 23, 4, 21, 2)], self.result.all())
+
+    def test_leftsibling_in_level(self):
+        """ level           Nested sets example
+            1                    1(1)22
+                    _______________|___________________
+                   |               |                   |
+            2    2(2)5           6(4)11             12(7)21
+                   |               ^                   ^
+            3    3(3)4       7(5)8   9(6)10    13(8)16   17(10)20
+                                                  |          |
+            4                                  14(9)15   18(11)19
+        """
+        node3 = self.session.query(Tree).filter(Tree.id == 3).one()
+        node5 = self.session.query(Tree).filter(Tree.id == 5).one()
+        node6 = self.session.query(Tree).filter(Tree.id == 6).one()
+        node8 = self.session.query(Tree).filter(Tree.id == 8).one()
+        node10 = self.session.query(Tree).filter(Tree.id == 10).one()
+        self.assertEqual(node10.leftsibling_in_level().id, node8.id)
+        self.assertEqual(node8.leftsibling_in_level().id, node6.id)
+        self.assertEqual(node6.leftsibling_in_level().id, node5.id)
+        self.assertEqual(node3.leftsibling_in_level(), None)
